variable "aws_profile" {
  type    = string
  default = "packer-demo"
  description = "AWS CLI profile with credentials and access for Packer to build in AWS."
}

variable "region" {
  type    = string
  default = "us-east-1"
  description = "AWS region where Packer is going to find the base image, build, and deploy the AMI."
}

variable "vulnerability_rule_name" {
  type    = string
  default = "packer-host-vulnerability-rule"
  description = "Name of the vulnerability rule in Prisma Cloud"
}

variable "compliance_policy_name" {
  type    = string
  default = "packer-host-compliance-policy"
  description = "Name of the compliance rule in Prisma Cloud"
}

variable "scratch_dir" {
  type    = string
  default = "/var/tmp/scan"
  description = "Temporary directory for file downloads, script execution, and scan results."
}

variable "subnet_id" {
  type    = string
  default = ""
  description = "Specify the subnet to deploy the build instance - leave blank for default VPC"
}

variable "vpc_id" {
  type    = string
  default = ""
  description = "Specify the VPC to deploy the build instance - leave blank for default VPC"
}

variable "show_scan_output" {
  type = string
  default = "no"
  description = "Set to 'yes' to see the full results of the host scan in the build log"
}

locals {
    app_name = "packer-demo"
    console_url = vault("/secret/data/prisma_cloud", "console_url")
    api_key = vault("/secret/data/prisma_cloud", "api_key")
    server_name = "packer-demo-{{ timestamp }}"
}

local "api_pass" {
    expression = vault("/secret/data/prisma_cloud", "api_secret")
    sensitive  = true
}

data "amazon-ami" "latest-al2-image" {
    profile         = "${var.aws_profile}"
    region      = "${var.region}"
    filters = {
        virtualization-type = "hvm"
        name = "amzn2-ami-hvm-*-x86_64-ebs"
        root-device-type = "ebs"
    }
    owners = ["amazon"]
    most_recent = true
}

source "amazon-ebs" "golden-image" {
  profile               = "${var.aws_profile}"
  ami_name      = "${local.app_name}-{{ timestamp }}"
  instance_type = "t3.micro"
  region        = "${var.region}"
  source_ami    = "${data.amazon-ami.latest-al2-image.id}"
  ssh_username  = "ec2-user"
  subnet_id             = "${var.subnet_id}"
  vpc_id                = "${var.vpc_id}"
}

build {
  sources = ["source.amazon-ebs.golden-image"]

  # execute the golden image build
  provisioner "shell" {
    script = "script/custom-build-script.sh"
  }

  # reset the hostname and reboot
  provisioner "shell" {
    inline = [ "sudo hostnamectl set-hostname ${local.server_name} && sudo reboot" ]
    expect_disconnect = true
  }

  # scan for vulnerability and compliance
  provisioner "shell" {
        environment_vars = [
                "PRISMA_CLOUD_USER=${local.api_key}",
                "PRISMA_CLOUD_PASS=${local.api_pass}",
                "PRISMA_CLOUD_CONSOLE_URL=${local.console_url}",
		"PRISMA_CLOUD_VULNERABILITY_RULE_NAME=${var.vulnerability_rule_name}",
		"PRISMA_CLOUD_COMPLIANCE_POLICY_NAME=${var.compliance_policy_name}",
		"BUILD_HOSTNAME=${local.server_name}",
                "SCRATCH_DIR=${var.scratch_dir}",
		"SHOW_SCAN_OUTPUT=${var.show_scan_output}"
        ]

        script = "script/download-scan-check.sh"

  }
}
