#!/usr/bin/env bash
# Written by Kyle Butler

TL_USER="$PRISMA_CLOUD_USER"
TL_PASSWORD="$PRISMA_CLOUD_PASS"
TL_CONSOLE="$PRISMA_CLOUD_CONSOLE_URL"
VULN_RULE_NAME="$PRISMA_CLOUD_VULNERABILITY_RULE_NAME"
COMPLIANCE_RULE_NAME="$PRISMA_CLOUD_COMPLIANCE_POLICY_NAME"
TL_TEMP_FOLDER="$SCRATCH_DIR"
BUILD_HOSTNAME="$BUILD_HOSTNAME"
SHOW_FULL_SCAN_OUTPUT="$SHOW_SCAN_OUTPUT"

### END OF USER CONFIG ###

# make sure the scratch dir exists
if [ ! -d "${TL_TEMP_FOLDER}" ]; then
  mkdir -p "${TL_TEMP_FOLDER}" || exit 1
fi

echo "checking to see if jq is installed and installing if not..."
# installs jq based on linux distro
if ! command -v jq &> /dev/null; then
  touch "${TL_TEMP_FOLDER}/.removejq"
  if [ -f /etc/os-release ]; then
      # freedesktop.org and systemd
      source /etc/os-release
      case "$ID" in
          ubuntu | debian)
              # Ubuntu/Debian
              sudo apt-get update
              sudo apt-get install -y jq
              ;;
          talos)
              # Talos OS
              sudo curl -s -L "https://github.com/stedolan/jq/releases/latest/download/jq-linux64" -o /usr/local/bin/jq
              sudo chmod +x /usr/local/bin/jq
              ;;
          sles | opensuse | suse)
              # SUSE/OpenSUSE
              sudo zypper install -y jq
              ;;
          rockylinux | rhcos | rhel | oel | centos)
              # Rocky Linux/RHCOS/RHEL/OEL/CentOS
              sudo yum install -y jq
              ;;
          amzn | amzn2)
              # Amazon Linux/Amazon Linux 2
              #sudo amazon-linux-extras install -y epel
	      #sudo yum update
              sudo yum install -y jq
              ;;
          *)
              echo "Unsupported distribution: $ID"
              exit 1
              ;;
      esac
  else
      printf '\n%s\n' "Unable to determine the Linux Distribution"
      exit 1
  fi
fi

# make sure we installed jq
if ! command -v jq &> /dev/null; then
  echo "Unable to find jq.  Exiting." && exit 1
fi

AUTH_PAYLOAD=$(cat <<EOF
{"username": "$TL_USER", "password": "$TL_PASSWORD"}
EOF
)

echo "Authenticating to Prisma Compute Console"

PRISMA_COMPUTE_API_AUTH_RESPONSE=$(curl -s \
					--header "Content-Type: application/json" \
                                        --request POST \
                                        --data "$AUTH_PAYLOAD" \
                                        --url "$TL_CONSOLE/api/v1/authenticate" )

TL_JWT=$(printf '%s' "$PRISMA_COMPUTE_API_AUTH_RESPONSE" | jq -r '.token')

HOSTNAME_FOR_CONSOLE=$(printf '%s' "$TL_CONSOLE" | awk -F / '{print $3}' | sed  s/':\S*'//g)

echo "Installing linux host defender"
curl -sSL \
     --header "authorization: Bearer $TL_JWT" \
     --request POST \
     --url "$TL_CONSOLE/api/v1/scripts/defender.sh" | sudo bash -s -- -c "$HOSTNAME_FOR_CONSOLE" -d "none" -m --install-host

echo "Scanning Host please wait..."
# sleep while scan happens
sleep 30

echo "Retrieving results from Prisma Compute Console.."
curl -s \
     --header "Authorization: Bearer $TL_JWT" \
     --header 'Content-Type: application/json' \
     --request GET \
     --url "$TL_CONSOLE/api/v1/hosts?hostname=$BUILD_HOSTNAME" \
     -o "$TL_TEMP_FOLDER/scan-results.json"

echo "Retrieving the Vulnerability Policy..."
curl -s \
     --header "Authorization: Bearer $TL_JWT" \
     --header 'Content-Type: application/json' \
     --request GET \
     --url "$TL_CONSOLE/api/v1/policies/vulnerability/host" \
     -o "$TL_TEMP_FOLDER/vulnerability-policy.json"

echo "Retrieving the Compliance Policy"
curl -s \
     --header "Authorization: Bearer $TL_JWT" \
     --header 'Content-Type: application/json' \
     --request GET \
     --url "$TL_CONSOLE/api/v1/policies/compliance/host?hostname=$BUILD_HOSTNAME" \
     -o "$TL_TEMP_FOLDER/compliance-policy.json"

COMPLIANCE_BLOCK_ARRAY=( $(jq --arg compliance_rule_name "$COMPLIANCE_RULE_NAME" '.rules[] | select(.name == $compliance_rule_name ) | .condition.vulnerabilities[] | select( .block == true) | .id' "$TL_TEMP_FOLDER/compliance-policy.json") )
COMPLIANCE_ISSUE_ARRAY=($( jq '.[].complianceIssues[].id' "$TL_TEMP_FOLDER/scan-results.json" ))

COMP_EXIT_CODE=0
for compliance_issue in "${!COMPLIANCE_ISSUE_ARRAY[@]}"; do \
  if [[ " ${COMPLIANCE_BLOCK_ARRAY[*]} "  =~ " ${COMPLIANCE_ISSUE_ARRAY[$compliance_issue]} " ]]; then
    COMPLIANCE_TITLE=$(jq --arg comp_id ${COMPLIANCE_ISSUE_ARRAY[$compliance_issue]} -r '.[].complianceIssues[] | select( .id == ( $comp_id | tonumber) ) | .title' "$TL_TEMP_FOLDER/scan-results.json")
    echo "Compliance violation found. Compliance ID:  ${COMPLIANCE_ISSUE_ARRAY[$compliance_issue]} Policy Title: ${COMPLIANCE_TITLE}"
    COMP_EXIT_CODE=1
  fi
done

if [ "$COMP_EXIT_CODE" -eq 0 ]; then
  echo "no compliance violations found"
fi

###VULNERABILITY THRESHOLD INFO
# critical = 9
# hith + critical = 7
# medium + high + critical = 4
# low + medium + high + critical = 1
# off 0
VULN_THRESHOLD=$(jq --arg vuln_rule_name "$VULN_RULE_NAME" '.rules[] | select(.name == $vuln_rule_name) | .alertThreshold.value' "$TL_TEMP_FOLDER/vulnerability-policy.json")

# check the vulnerability results
VUL_EXIT_CODE=0
if [ "$VULN_THRESHOLD" == "0" ]; then
  echo "no vulnerability block threshold in place"
else
  #parse the scan results file
  VULNCRITICAL=$( jq --arg build_hostname "$BUILD_HOSTNAME" '.[] | select( ._id == $build_hostname) | .vulnerabilityDistribution.critical' "$TL_TEMP_FOLDER/scan-results.json" )
  VULNHIGH=$( jq --arg build_hostname "$BUILD_HOSTNAME" '.[] | select( ._id == $build_hostname ) | .vulnerabilityDistribution.high' "$TL_TEMP_FOLDER/scan-results.json" )
  VULNMED=$( jq --arg build_hostname "$BUILD_HOSTNAME" '.[] | select( ._id == $build_hostname ) | .vulnerabilityDistribution.medium' "$TL_TEMP_FOLDER/scan-results.json" )
  VULNLOW=$( jq --arg build_hostname "$BUILD_HOSTNAME" '.[] | select( ._id == $build_hostname ) | .vulnerabilityDistribution.low' "$TL_TEMP_FOLDER/scan-results.json" )
  sev_array="[]"

  if [ "$VULN_THRESHOLD" == "9" ]; then
    VULNRESULT="$VULNCRITICAL"
    if [ "${VULNRESULT}" -ne "0" ]; then
      sev_array='["critical"]'
      VUL_EXIT_CODE=1
    else
      echo "no critical vulnerabilities were found"
      VUL_EXIT_CODE=0
    fi
  elif [ "$VULN_THRESHOLD" == "7" ]; then
    VULNRESULT=$(( $VULNCRITICAL + $VULNHIGH ))
    if [ "${VULNRESULT}" -ne "0" ]; then
      sev_array='["critical","high"]'
      VUL_EXIT_CODE=1
    else
      echo "no critical or high vulnerabilities were found"
      VUL_EXIT_CODE=0
    fi
  elif [ "$VULN_THRESHOLD" == "4" ]; then
    VULNRESULT=$(( $VULNCRITICAL + $VULNHIGH + $VULNMED ))
    if [ "${VULNRESULT}" -ne "0" ]; then
      sev_array='["critical","high","medium"]'
      VUL_EXIT_CODE=1
    else
      echo "no critical, high, or medium vulnerabilities were found"
      VUL_EXIT_CODE=0
    fi
  elif [ "$VULN_THRESHOLD" == "1" ]; then
    VULNRESULT=$(( $VULNCRITICAL + $VULNHIGH + $VULNMED + $VULNLOW ))
    if [ "${VULNRESULT}" -ne "0" ]; then
      sev_array='["critical","high","medium","low"]'
      VUL_EXIT_CODE=1
    else
      echo "no critical, high, medium, or low vulnerabilities were found"
      VUL_EXIT_CODE=0
    fi
  else
    echo "unexpected result from Prisma Cloud vulnerability rule" && exit 1
  fi

  if [ "$VUL_EXIT_CODE" -eq 1 ]; then
    echo "build failed with the following vulnerabilities:"
    cat "$TL_TEMP_FOLDER/scan-results.json" | jq -r --argjson severities "$sev_array" '["CVE", "SEVERITY", "DESCRIPTION"],["___", "________", "___________"], (.[].vulnerabilities[] | select( [.severity] | inside( $severities ) ) | [.cve, .severity, .description] ) | @tsv'
  fi
fi


# print out the scan results if requested
if [ "$SHOW_FULL_SCAN_OUTPUT" == "yes" ]; then
  echo "Results of the host scan:"
  cat "$TL_TEMP_FOLDER/scan-results.json" | jq .
fi

# clean-up
echo "removing jq if it was previously installed"
if [ -f "${TL_TEMP_FOLDER}/.removejq" ]; then
  if [ -f /etc/os-release ]; then
      # freedesktop.org and systemd
      source /etc/os-release
      case "$ID" in
          ubuntu | debian)
              # Ubuntu/Debian
              sudo apt-get remove -y jq
              ;;
          talos)
              # Talos OS
              sudo rm -f /usr/local/bin/jq
              ;;
          sles | opensuse | suse)
              # SUSE/OpenSUSE
              sudo zypper remove -y jq
              ;;
          rockylinux | rhcos | rhel | oel | centos)
              # Rocky Linux/RHCOS/RHEL/OEL/CentOS
              sudo yum autoremove -y jq
              ;;
          amzn | amzn2)
              # Amazon Linux/Amazon Linux 2
              # sudo amazon-linux-extras remove -y epel
              sudo yum autoremove -y jq
              ;;
          *)
              echo "Unsupported distribution: $ID"
              exit 1
              ;;
      esac
  else
      printf '\n%s\n' "Unable to determine the Linux Distribution"
      exit 1
  fi
fi

echo "removing temp folder"
rm -rf "$TL_TEMP_FOLDER"

# remove prisma cloud host defender
echo "removing Prisma Cloud Defender"
sudo /var/lib/twistlock/scripts/twistlock.sh -u defender-server
curl -s "$TL_CONSOLE/api/v1/defenders/$BUILD_HOSTNAME?project=Central+Console" \
  -X 'DELETE' \
  -H 'Accept: application/json, text/plain, */*' \
  -H 'Accept-Language: en-US,en;q=0.7' \
  -H "Authorization: Bearer $TL_JWT"  \
  --compressed

EXIT_CODE=$(( $VUL_EXIT_CODE + $COMP_EXIT_CODE ))
if [ "$EXIT_CODE" -gt 0 ]; then
  exit 1
fi

exit 0; 
